<?php

/**
 * Project:     Collator: the Question Writer results collator
 * File:        results.php
 *
 * Copyright 2008-2009 Central Question Ltd.
 * 
 * This application is provided as an extension of the 
 * Question Writer desktop software. Where you hold a valid, paid-for
 * license for the Question Writer desktop software, you are hereby 
 * granted a license to use and modify this application for personal use
 * and use internal to your organization. All other rights are reserved. 
 * 
 * This application is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * @link http://www.questionwriter.com/
 * @copyright 2008-2009 Central Question Ltd.
 * @author Alexander McCabe 
 * @package Collator
 * @version 1.3
 * 
 * 1.3 
 * Changed Error Logging to remove uninteresting errors
 * 
 * 1.2
 * Added additional identifiers to the email where they are not blank
 * Added change IP recorded when resultset comes from backup server
 * Added additional Candidate fields written into userdata db fields
 * Added fix to XML parsing to allow for special XML characters in ID, also remove spaces after special XML characters
 * Added facility to check if e-mail is not provided directly, but referenced by an id
 * 
 * 1.1.1
 * Put all the Resultsets into a single SQL query to reduce processing time
 * Added ReplyTo header to help with spam rating
 * Added MIME header for e-mail to avoid MIME_HEADER_CTYPE_ONLY rating
 * Added newline in table for each line - email was being rejected for having lines that were too long
 * Externalized e-mail call
 * Added duplicate result checking in LogXML function
 * Set LogXML to false as a default
 * Check PassFail value for PASS/FAIL value - set to pass if value not recognized
 * Check for empty name/value pairs generated by old QW quizzes
 * 
 * 1.1.0
 * Added all questiondata to a single SQL statement
 * Added codedpreferences.inc.php to allow code to check whether to store results in database and send email
 * Time field now entered properly in QuestionData table after 'PS' characters trimmed
 * Removed spaces at start and end of PHP files which may have caused problems returning 500 errors
 * Added Total Time Field to database and filled it by summing time taken for questions
 * Added error returns where MySQL statements fail
 * Added Email functionality
 * Changed all table names to lower-case
 * Added logging of result set XML to files capability
 * Remove printing trivial XML document 'ok' from response
 * Formatted Quiz and Question title to remove extraneous numbers and ids
 * Modified total time to display in Hours, Minutes, Seconds
 * 
 * 1.0.0
 * Initial Release
 */

	//Include the files this script depends upon
	include "config.php";
	include "include/functions.inc.php";
	include "include/codedpreferences.inc.php";	
	include "include/parse_xml.inc.php";
	

	//Get the IP address of the result set sender
	$ipvar=$_SERVER['REMOTE_ADDR'];
	
	//Trim the e-mail to ignore leading and trailing whitespace.
	$emailID=trim($_GET['email']);
	
	//Ensure that the e-mail is a real email address.
	//This also returns an error if there are any disallowed characters in the email address
	if(!isValidEmail($emailID)) 
	{
		//error_log($_GET['id']);
		//Check if the user has been reference by ID instead
		$userid=trim($_GET['id']);
		
		if($emailID=getEmailFromUserID($userid,$dbhost,$dbusername,$dbuserpass,$db_name)){
			//Fine
		}else{
			//Otherwise return an error to the client		
			returnError("Invalid E-mail Address. EmailID:".trim($_GET['email'])." UserID:".trim($_GET['id']));
		}
		
	}

	//Set the timestamp
	$newTime=time();
	
	//Check if this is from the backup server in which case we use the timestamp of the backup server
	//We only accept the communicated timestamp if we confirm the IP is that of the backup server  
	if(isset($_GET['newtime']) && $ipvar==$backupServerIP)
	{
		$newTime=(int)$_GET['newtime'];
		$ipvar=$_GET['originalip'];
	}
		
	$HTTP_RAW_POST_DATA="";
	$newdatafile="";
	
	//Check there is a valid XML file in the POST request
	if($xml = getPostedXML())
	{		
		$data=$xml;
		//error_log($data);
		$newdatafile=$data;
		if (!xml_parse($xmlparser, $data)) 
		{
			// If XML is not valid, return error
			$reason = xml_error_string(xml_get_error_code($xmlparser));
			$reason .= xml_get_current_line_number($xmlparser);
			returnError("Error parsing posted XML".$reason);
		}
	}
	else
	{		
	  //If XML is not found, return error, but don't log
	  returnErrorNoLog();
	  //returnError("Error - did not find suitable XML " . $xml);	  
	}
	
	$XMLFileLog="";
	$XMLProcessedFileLog="";
	if(logXML()){
		
		//Check directory exists for day in question
		$XMLLogDirectory=$archivelogdirectory."/".date("Ymd",$newTime);
		if(!is_dir($XMLLogDirectory)){
			mkdir($XMLLogDirectory);
			chmod($XMLLogDirectory, 0757);
		}
		
		$md5hash=md5($newdatafile);
		
		//Write XML to directory
		$XMLFileLog=$XMLLogDirectory."/".$ipvar."-".$md5hash."-".$emailID.".xml";
		$XMLProcessedFileLog=$XMLLogDirectory."/".$ipvar."-".$md5hash."-".$emailID.".processed.xml";
		
		//Check for duplicates - exit with a processed code if this file has already been received.
		if(file_exists($XMLProcessedFileLog)){
			header('Content-Type: text/xml; Charset: utf-8');
			print("<OK/>");
			exit();
		}
		
		if($handle1=fopen($XMLFileLog, "w")){
			if (fwrite($handle1, $newdatafile)){
				if(!fclose($handle1)){
					returnError("Error - could not close log file - "+$XMLFileLog);
				}
			}else
			{
				returnError("Error - could not write XML to log file"+$XMLFileLog);
			}
		}else{
			returnError("Error - could not open file for writing - "+$XMLFileLog);
		}
	}
	
	
	$Ascores['Email']=$emailID;
	$Ascores['TimeStamp']=$newTime;
	$Ascores['QuizName']=$quizreference;
	$Ascores['UserIP']=$ipvar;
	$Ascores['TotalTime']=$totaltime;
	$Ascores['Data1']=$userdata1;
	$Ascores['Data2']=$userdata2;
	$Ascores['Data3']=$userdata3;
	$Ascores['Data4']=$userdata4;
	$Ascores['Data5']=$userdata5;
	
	//Check pass/fail value is PASS or FAIL
	if($Ascores['PassFail']!='PASS' && $Ascores['PassFail']!='FAIL'){
		$Ascores['PassFail']='PASS';
	} 
	
	//Check if the user has requested not to store info in the database
	if(storeResultsInDatabase($emailID,$dbhost,$dbusername,$dbuserpass,$db_name)){
				
		//Open the database connection
		mysql_connect($dbhost,$dbusername,$dbuserpass) or returnError("Error - could not connect to database");
		
		//Choose the database
		@mysql_select_db($db_name) or returnError("Error - unable to select database");
		
		//Begin the transaction - we'll only write to the database if we successfully write all the questions to the database too
		mysql_query("BEGIN") or returnError("Error - unable to begin database transaction");
		
		//Write the resultset data to the database
		$Ascores['ResultSetID']="";
				
		$query = generateSafeMySQLQuery('resultsets',$Ascores);
		//error_log($query);
		mysql_query($query) or returnError("Error - unable to write result set data");
		

		//Get the updated ID of the resultset
		$resultid=mysql_insert_id();
		
		//Write each of the questions to the question data table 	
		$countques=count($Aanswers);
		
		$query="INSERT INTO questiondata (ResultSetID,SectionReference,QuestionReference,Response,LongResponse,Score,QuestionTime) VALUES ";
		for($x=0;$x<$countques;$x++){
			
			$query .= "(";
			//$Anewques[QuestionID]="";
			
			$query .= "\"".mysql_real_escape_string($resultid)."\",";
			//$Anewques[ResultSetID]=$resultid; //SET RESULT ID
			
			$query .= "\"".mysql_real_escape_string($Aanswers[$x][0])."\",";
			//$Anewques[SectionReference]=$Aanswers[$x][0]; //SET SECTION REFERENCE
			
			$query .= "\"".mysql_real_escape_string($Aanswers[$x][5])."\",";
			//$Anewques[QuestionReference]=$Aanswers[$x][5]; //SET QUESTION REFERENCE
						
			if(strlen($Aanswers[$x][2])<255) //IF ANSWER IS SHORT
			{
				$query .= "\"".mysql_real_escape_string($Aanswers[$x][2])."\",";
				$query .= "\"\",";
				//$Anewques[Response]=$Aanswers[$x][2]; //SET TO SAVE IN MAIN RESULT TABLE
			}
			else //IF ANSWER IS ESSAY LENGTH
			{
				$query .= "\"\",";
				$query .= "\"".mysql_real_escape_string($Aanswers[$x][2])."\",";
				//$Anewques[LongResponse]=$Aanswers[$x][2];
			}//END IF
			
			$query .= "\"".mysql_real_escape_string($Aanswers[$x][3])."\",";
			//$Anewques[Score]=$Aanswers[$x][3]; //SET SCORE
			
			
			//Trim the P and S from the time field
			$query .= "\"".mysql_real_escape_string(trim($Aanswers[$x][4],"PS"))."\"),";
			//$Anewques[QuestionTime]= trim($Aanswers[$x][4],"PS");//SET TIME TAKEN FOR QUESTION 
			
			//$query = generateSafeMySQLQuery('questiondata',$Anewques);
			
			
		}//END FOR..
		$query=trim($query,",");
		//error_log($query);
		mysql_query($query) or returnError("Error - unable to write question data");


		
		//Commit the transaction, writing to the database and closing the connection
		mysql_query("COMMIT");

		mysql_close();
	}	
	
	//Check if the user has requested email
	if(sendResultsByEmail($emailID,$dbhost,$dbusername,$dbuserpass,$db_name)){
				
		//Main Email Body
		$mailclient="";
		$mailclient.="<h1>".htmlspecialchars(formatQuizTitle($Ascores['QuizName']))."</h1>". "\r\n";
		$mailclient.="<h2>Summary</h2>". "\r\n";
		$mailclient.="<table border=\"0\" cellspacing=\"0\" cellpadding=\"4\">". "\r\n";
		$mailclient.="  <tr><th align=\"right\">Name or User ID: </th><td>".htmlspecialchars($Ascores['Candidate'])."</td></tr>". "\r\n";
		
		if(strlen($Ascores['Data2'])>0){
			$mailclient.="  <tr><th align=\"right\">Second Identifier: </th><td>".htmlspecialchars($Ascores['Data2'])."</td></tr>". "\r\n";
		}
		if(strlen($Ascores['Data3'])>0){
			$mailclient.="  <tr><th align=\"right\">Third Identifier: </th><td>".htmlspecialchars($Ascores['Data3'])."</td></tr>". "\r\n";
		}
		if(strlen($Ascores['Data4'])>0){
			$mailclient.="  <tr><th align=\"right\">Fourth Identifier: </th><td>".htmlspecialchars($Ascores['Data4'])."</td></tr>". "\r\n";
		}
		if(strlen($Ascores['Data5'])>0){
			$mailclient.="  <tr><th align=\"right\">Fifth Identifier: </th><td>".htmlspecialchars($Ascores['Data5'])."</td></tr>". "\r\n";
		}
		
		$mailclient.="  <tr><th align=\"right\">Overall Score: </th><td>".htmlspecialchars($Ascores['OverAllScore'])."</td></tr>". "\r\n";
		$mailclient.="  <tr><th align=\"right\">Total Possible Score: </th><td>".htmlspecialchars($Ascores['TotalPossibleScore'])."</td></tr>". "\r\n";
		$mailclient.="  <tr><th align=\"right\">Percentage Score: </th><td>".htmlspecialchars($Ascores['PercentageScore'])."</td></tr>". "\r\n";
		$mailclient.="  <tr><th align=\"right\">Percentage Score String: </th><td>".htmlspecialchars($Ascores['PercentageScoreString'])."</td></tr>". "\r\n";
		$mailclient.="  <tr><th align=\"right\">Percentage Pass: </th><td>".htmlspecialchars($Ascores['PercentagePass'])."</td></tr>". "\r\n";
		$mailclient.="  <tr><th align=\"right\">Pass/Fail: </th><td>".htmlspecialchars($Ascores['PassFail'])."</td></tr>". "\r\n";
		$mailclient.="  <tr><th align=\"right\">Timestamp: </th><td>".htmlspecialchars(date("D, M j, G:i:s T Y",$Ascores['TimeStamp']))."</td></tr>". "\r\n";
		$mailclient.="  <tr><th align=\"right\">Time Spent: </th><td>".htmlspecialchars(formatseconds(number_format($totaltime,2,'.','')))."</td></tr>". "\r\n";
		$mailclient.="</table><h2>Questions</h2>". "\r\n";
		$mailclient.="<table border=\"0\" cellspacing=\"0\" cellpadding=\"4\">". "\r\n";
		$mailclient.="<tr><th>Question</th><th>Score</th><th>Duration</th><th></th><th>Answer</th></tr>". "\r\n";
		$countques=count($Aanswers);
		for($x=0;$x<$countques;$x++)
		{
			$mailclient.="<tr><td>".htmlspecialchars(formatQuestionTitle($Aanswers[$x][1]))."</td><td align=\"right\">".htmlspecialchars($Aanswers[$x][3])."</td><td align=\"right\">".htmlspecialchars(number_format(trim($Aanswers[$x][4],"PS"),2,'.',''))."s</td><td>&nbsp;</td><td>".nl2br(htmlspecialchars($Aanswers[$x][2]))."</td></tr>\r\n";
		}//END FOR..
		$mailclient.="</table>". "\r\n";
		$mailclient.="<br/>Quiz sent ".htmlspecialchars($Ascores['Timestamp'])." from IP ".htmlspecialchars($ipvar). "<br/>http://www.networldmap.com/TryIt.htm?GetLocation&ipaddress=".htmlspecialchars($ipvar)."\r\n";
					
		$subject="Quiz Results ".htmlspecialchars($Ascores['Candidate'])."- ".htmlspecialchars($Ascores['OverAllScore'])." (".htmlspecialchars($Ascores['PercentageScore'])."%)";
		
		sendEmail($emailID, $subject, $mailclient, $from, $fromemail,$dbhost,$dbusername,$dbuserpass,$db_name);
		
	}
	
	//Rename the log file to indicate that it has been processed.
	//This is help prevent duplicate requests being reprocessed.
	if(logXML() && file_exists($XMLFileLog) && !file_exists($XMLProcessedFileLog)){
		rename($XMLFileLog,$XMLProcessedFileLog);
	}
	
	//Return suitable non-error to client
	header('Content-Type: text/xml; Charset: utf-8');
	print("<OK/>");
	//End of script
?>